#!/bin/perl
#############################################################
# WebSphere Usage Statistics
# =========================
# Copyright (C) 2013
# =========================
# Description: 
#  The program gathers usage statistics from WebSphere
#  and places them under node "JVM Stats"
#  Information displayed will not be by WAS appname due to 
#  'ps' output limitations by Solaris.
# =========================
# Usage: perl psWASforSolaris.pl
#############################################################
use FindBin;
use lib ("$FindBin::Bin", "$FindBin::Bin/lib/perl", "$FindBin::Bin/../lib/perl");
use Wily::PrintMetric;

use strict;

my $psCommand='ps -efo pid,pcpu,pmem,vsz,rss,comm|grep was64';
my @psResults=`$psCommand`;

# output results
#PID %CPU %MEM VSZ RSS COMM	# HEADER IS NOT DISPLAYED	
#98808  0.1  1.1 4939536 2921944 /opt/webapps/was64/base/as/java/bin/sparcv9/java
#18230  0.0  1.1 3992056 2878872 /opt/webapps/was64/base/as/java/bin/sparcv9/java

#pid: proc id
#pcpu: The ratio of CPU time used recently to CPU time available in the same period, expressed as a percentage.
# The meaning of "recently" in this context is unspecified.
#vsz: The total size of the process in virtual memory, in kilobytes.
#rss: The resident set size of the process, in kilobytes.
#comm: The name of the command being executed (argv[0] value) as a string.

# initialize counter for total resident memory
my $total=0;

#initialize counter for JVMs
my $jvm=1;

# parse results
foreach my $line (@psResults){
	chomp $line;
	# split the row
	my @jvmStats=split(/\s+/,$line);
	# increment RSS counter
	($total+=$_) for $jvmStats[4];
	# print metrics
	Wily::PrintMetric::printMetric(	type		=> 'IntCounter',
									resource	=> 'JVM Stats',
									subresource	=> 'JVM' . $jvm,
									name		=> 'RSS',
									value		=> int($jvmStats[4]),
								  );
	Wily::PrintMetric::printMetric(	type		=> 'IntCounter',
									resource	=> 'JVM Stats',
									subresource	=> 'JVM' . $jvm,
									name		=> 'VSZ',
									value		=> int($jvmStats[3]),
								  );
	Wily::PrintMetric::printMetric(	type		=> 'StringEvent',
									resource	=> 'JVM Stats',
									subresource	=> 'JVM' . $jvm,
									name		=> 'MEM (%)',
									value		=> $jvmStats[2],
								  );
	Wily::PrintMetric::printMetric(	type		=> 'StringEvent',
									resource	=> 'JVM Stats',
									subresource	=> 'JVM' . $jvm,
									name		=> 'CPU (%)',
									value		=> $jvmStats[1],
								  );
	Wily::PrintMetric::printMetric(	type		=> 'StringEvent',
									resource	=> 'JVM Stats',
									subresource	=> 'JVM' . $jvm,
									name		=> 'PID',
									value		=> $jvmStats[0],
								  );
	$jvm++;
}
# print the total resident memory usage
Wily::PrintMetric::printMetric(	type		=> 'LongCounter',
								resource	=> 'JVM Stats',
								name		=> 'Total Resident Memory (KB)',
								value		=> $total,
							  );
